/**
 * Title:        Report Writer Example
 * Version:      1.0, 30/11/2001
 * Description:  A Grid Scheduler for Application Scheduling on Grid based on
 *               Deadline and Budget Constrained Scheduling Algorithms
 * @author       R. Buyya and M. Murshed (by surname)
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

import java.util.*;
import java.io.*;
import eduni.simjava.*;
import gridsim.*;

/**
 * A user defined Report Writter Entity.
 */
class ReportWriter extends GridSim {

  private int no_of_users;
  private int no_of_resources;
  private String filename;

  public ReportWriter(String name, int no_of_users, int no_of_resources, String filename)
  {
    super(name);
    this.no_of_users = no_of_users;
    this.no_of_resources = no_of_resources;
    this.filename = filename;
  }

  public void body()
  {
    Sim_event ev = new Sim_event();

    // Wait for END_OF_SIMULATION
    for( sim_get_next(ev);  ev.get_tag() != GridSimTags.END_OF_SIMULATION; sim_get_next(ev) )
      System.out.println(get_name()+" -Unwanted event!");

    GenerateReport();

    // GridStatistics should be shutdown after generating report
    ShutdownGridStatisticsEntity();
  }

  private void GenerateReport() {
    try {
      FileWriter fw = new FileWriter(filename, true);
      BufferedWriter bw = new BufferedWriter(fw);
      PrintWriter outFile = new PrintWriter(bw);

      outFile.print(no_of_users+"\t"+no_of_resources);

      String[] category = {"*.USER.TimeUtilization", "*.USER.BudgetUtilization"};
      for(int i = 0; i < category.length; i++) {
        Sim_event ev = new Sim_event();
        sim_schedule(GridSim.GetEntityId("GridStatistics"), GridSimTags.SCHEDULE_NOW, GridSimTags.RETURN_ACC_STATISTICS_BY_CATEGORY, category[i]);
        sim_get_next(ev);
        Accumulator acc = (Accumulator) ev.get_data();
        outFile.print("\t"+acc.mean());
      }
      outFile.println();

      outFile.close();
    }
    catch(IOException e) {
      System.out.println(get_name()+"-Unable to open report file: "+filename);
    }
  }
}