/**
 * Title:        GridSim Toolkit
 * Version:      GridSim  1.0, 12/09/2001
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation of
 *               Parallel and Distributed Systems such as Clusters and Grids
 * @author       R. Buyya and M. Murshed (by surname)
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

package gridsim;

import java.util.*;
import java.io.*;
import eduni.simjava.*;

/**
 * A calender that resource needs for advance booking of resources
 * and also deciding standard load pattern.
 */
class ResourceCalendar {
  private Random r;
  private double TimeZone;
  private double[] WeekdayLoad;
  private double[] HolidayLoad; // includes weekends as well

  private LinkedList weekends;
  private LinkedList holidays;

/*  class ResourceBooking {
    private int user_id;
    private Date StartTime;
    private Date EndTime;
    private int machine_id;
    private int PE_id;
    private Task t;

    public ResourceBooking(int user_id, Date StartTime, Date EndTime,
                            int machine_id, int PE_id) {
      this.user_id = user_id;
      this.StartTime = StartTime;
      this.EndTime = EndTime;
      this.machine_id = machine_id;
      this.PE_id = PE_id;
    }
  }
*/
  private LinkedList bookings;

  // LinkedList weekend = a list of 0 = Sunday, 1 = Monday, 2 = Tuesday, 3 = Wednesday, 4 = Thursday, 5 = Friday, 6 = Saturday)
  public ResourceCalendar(double TimeZone, double peakLoad, double offpeakLoad,
                          double relativeHolidayLoad, LinkedList weekends,
                          LinkedList holydays, long seed) {
    r = new Random(seed);
    this.TimeZone = TimeZone;

    // initialised as per common observation of relative local usage behavior of resource
    double[] RegularLoad = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                            0.0, 0.1, 0.2, 0.4, 0.8, 1.0,
                            1.0, 0.6, 0.6, 0.9, 1.0, 1.0,
                            0.5, 0.2, 0.1, 0.0, 0.0, 0.0};

    WeekdayLoad = new double[24];
    HolidayLoad = new double[24];
    for(int i = 0; i < 24; i++) {
      double val = RegularLoad[i]*(peakLoad - offpeakLoad) + offpeakLoad;
      this.WeekdayLoad[i] = GridSimRandom.real(val, 0.1, 0.1, r.nextDouble());
      // relativeHolidayLoad is percentage of holiday load relative to weekday load
      this.HolidayLoad[i] = GridSimRandom.real(relativeHolidayLoad*val, 0.1, 0.1, r.nextDouble());
    }

    this.weekends = weekends;
    this.holidays = holidays;
  }

  public Calendar CalendarAtGivenSimTime(double SimulationTime) {
    Calendar calendar = Calendar.getInstance();
    Date d = GridSim.GetSimulationStartDate();

    // Set calendar time as the Simulation start date-time at 0:00:00 GMT
    calendar.setTime(d);

    // Adjust calendar time for time zone
    calendar.add(Calendar.MINUTE, (int) (TimeZone*60));

    calendar.add(Calendar.SECOND, (int) SimulationTime);

    return calendar;
  }

  public Calendar CalendarNow() {
    return CalendarAtGivenSimTime(Sim_system.clock());
  }

  public double SimulationTime(Calendar LocalTime) {
    LocalTime.add(Calendar.MINUTE, (int) (-TimeZone*60));
    Calendar Start = CalendarAtGivenDate(GridSim.GetSimulationStartDate());
    return (LocalTime.getTime().getTime() - Start.getTime().getTime())/1000.0;
  }

  public Calendar CalendarAtGivenDate(Date d) {
    Calendar calendar = Calendar.getInstance();
    calendar.setTime(d);

    // Adjust calendar time for time zone
    calendar.add(Calendar.MINUTE, (int) (TimeZone*60));

    return calendar;
  }

  public boolean IsHoliday() {
    Calendar myCalendar = CalendarNow();
    int day_of_year = myCalendar.get(Calendar.DAY_OF_YEAR);
    int day_of_week = myCalendar.get(Calendar.DAY_OF_WEEK);
    if(holidays!=null)
      if(holidays.contains(new Integer(day_of_year)))
        return true;
    if(weekends!=null)
      if(weekends.contains(new Integer(day_of_week)))
        return true;
    return false;
  }
  public boolean IsHoliday(Date d) {
    if(holidays==null)
      return false;
    Calendar myCalendar = CalendarAtGivenDate(d);
    myCalendar.setTime(d);
    int day_of_year = myCalendar.get(Calendar.DAY_OF_YEAR);
    int day_of_week = myCalendar.get(Calendar.DAY_OF_WEEK);
    if(holidays!=null)
      if(holidays.contains(new Integer(day_of_year)))
        return true;
    if(weekends!=null)
      if(weekends.contains(new Integer(day_of_week)))
        return true;
    return false;
  }

  public double CurrentLoad() {
    Calendar cal = CalendarNow();
    String str = cal.toString();
    if(IsHoliday())
      return HolidayLoad[CalendarNow().HOUR_OF_DAY];
    else
      return WeekdayLoad[CalendarNow().HOUR_OF_DAY];
  }
}
